@@ -1,9 +1,10 @@
+import logging
 import unittest
 import firebase_admin
 from firebase_admin import credentials
 
 import upswingutil as ul
-from upswingutil.pms.rms import ReservationSyncV2
+from upswingutil.pms.rms import ReservationSync
 from upswingutil.resource import access_secret_version, setup_logging
 import json
 
@@ -12,6 +13,7 @@ class TestRMSReservationV2API(unittest.TestCase):
     ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
     ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
     ul.MONGO_URI = access_secret_version(ul.G_CLOUD_PROJECT, 'MONGOURI', '1')
+    ul.LOG_LEVEL_VALUE = logging.DEBUG
 
     def test_adding_new_reservation(self):
         setup_logging()
@@ -27,9 +29,10 @@ class TestRMSReservationV2API(unittest.TestCase):
         cred = credentials.Certificate(cred)
         firebase_admin.initialize_app(cred, name='alvie')
 
-        resv = ReservationSyncV2('11249')
-        resv_list = [26506]
+        resv = ReservationSync('11249')
+        resv_list = [1, 28432]
         for _resv_id in resv_list:
             print(f'processing reservation : {_resv_id}')
             resv.process(_resv_id)
             print(f'processed reservation : {_resv_id}')
+
