@@ -40,6 +40,13 @@ class Mongodb:
             logger.error('Error while connecting to db')
             logger.error(e)
 
+    def create_view(self, view_name, view_on, pipeline):
+        return self.db.command({
+            "create": view_name,
+            "viewOn": view_on,
+            "pipeline": pipeline
+        })
+
     def execute_pipeline(self, name: str, pipeline: list):
         try:
             return list(self.db.get_collection(name).aggregate(pipeline))
