@@ -6,6 +6,8 @@ from upswingutil.schema import Token
 import upswingutil as ul
 import requests
 import logging
+import asyncio
+from concurrent.futures import ThreadPoolExecutor
 
 
 def _get_geo_region(nationalID: int):
