@@ -1,16 +1,16 @@
 import json
 from datetime import datetime
 
-from upswingutil.db.model import ReservationModel, Status, ReservationGuestInfo, ReservationGuestShort, GuestsModel, \
+from upswingutil.db.model import Status, ReservationGuestInfo, ReservationGuestShort, GuestsModel, \
     ReservationModelV2, GuestsModelV2
 from upswingutil.resource import get_model_from_cloud_storage
 from upswingutil.pms.rms import NAME, get_key, validate_key
 from upswingutil.db import Mongodb, Firestore, MongodbV2
 from upswingutil.schema import Token
-import upswingutil as ul
 import requests
 import logging
-
+import asyncio
+from concurrent.futures import ThreadPoolExecutor, as_completed
 
 def _get_status(status):
     if status == 'NoShow':
@@ -182,7 +182,10 @@ def _transform_guest_profile(record):
         },
         metaInfo={
             'notes': record.get('notes'),
-            'languageSpokenId': record.get('languageSpokenId')
+            'languageSpokenId': record.get('languageSpokenId'),
+            'contracts': record.get('contracts'),
+            'accounts': record.get('accounts'),
+            'mealCard': record.get('mealCard')
         },
         company={
           'companyId': record.get('companyId')
@@ -213,13 +216,29 @@ def _add_reservation_to_alvie(record):
     for email in user_email_list:
         docs = firestore_db.get_collection('users').where("email", "==", email).stream()
         for doc in docs:
-            logging.info("loading reservation to alvie")
+            logging.debug("loading reservation to alvie")
             firestore_db.get_collection(f'users/{doc.id}/reservations') \
                 .document(str(record.get("id"))) \
                 .set(resv_info, merge=True)
             logging.debug(f"added to {doc.id}")
 
 
+def _retrieve_data(session, id, name, url):
+    result = None
+    try:
+        logging.debug(f'Extracting {name} for reservation: {id}')
+        with session.get(url) as response:
+            if response.status_code == 200:
+                result = response.json()
+            else:
+                logging.debug(f'Error getting {name} for reservation {id} due to status code {response.status_code}')
+                logging.debug(url.format(id))
+    except Exception as e:
+        logging.error(f' {name} failed due to {e}')
+    finally:
+        return name, result
+
+
 class ReservationSync:
 
     def __init__(self, orgId: str, g_cloud_token=None):
@@ -278,7 +297,7 @@ class ReservationSync:
     def _retrieve_data(self, id, name, url):
         result = None
         try:
-            logging.info(f'Extracting {name} for reservation: {id}')
+            logging.debug(f'Extracting {name} for reservation: {id}')
             header = {
                 'Content-Type': 'application/json',
                 'authtoken': self.token.key
@@ -292,94 +311,99 @@ class ReservationSync:
         except Exception as e:
             logging.error(f' {name} failed due to {e}')
         finally:
-            return result
+            return name, result
 
     def extract_reservation_details(self, record):
         feature_list = [
             {
                 'name': 'holds',
-                'url': 'holds'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/holds"
             },
             {
                 'name': 'guests',
-                'url': 'guests'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/guests"
             },
             {
                 'name': 'billTo',
-                'url': 'billTo'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/billTo"
             },
             {
                 'name': 'transfers',
-                'url': 'transfers'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/transfers"
             },
             {
                 'name': 'auditTrail',
-                'url': 'auditTrail'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/auditTrail"
             },
             {
                 'name': 'daily_rates',
-                'url': 'dailyRates'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/dailyRates"
             },
             {
                 'name': 'rego_access',
-                'url': 'regoAccess'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/regoAccess"
             },
             {
                 'name': 'requirement',
-                'url': 'requirements'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/requirements"
             },
             {
                 'name': 'housekeeping',
-                'url': 'housekeeping'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/housekeeping"
             },
             {
                 'name': 'daily_revenue',
-                'url': 'dailyRevenue'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/dailyRevenue"
             },
             {
                 'name': 'add_ons',
-                'url': 'reservationAddOn'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/reservationAddOn"
             },
             {
                 'name': 'correspondence',
-                'url': 'correspondence'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/correspondence"
             },
             {
                 'name': 'financial_info_actual',
-                'url': 'actualAccount'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/actualAccount"
             },
             {
                 'name': 'bedConfiguration',
-                'url': 'bedConfiguration'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/bedConfiguration"
             },
             {
                 'name': 'existingToken',
-                'url': 'existingToken'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/existingToken"
             },
             {
                 'name': 'projectedAccount',
-                'url': 'projectedAccount'
+                'url': f"{self.token.hostName}/reservations/{record.get('id')}/projectedAccount"
+            },
+            {
+                'name': 'reservationTypes',
+                'url': f"{self.token.hostName}/reservationTypes/{record.get('resTypeId')}"
+            },
+            {
+                'name': 'marketSegment',
+                'url': f"{self.token.hostName}/marketSegments/{record.get('marketSegmentId')}"
+            },
+            {
+                'name': 'subMarketSegment',
+                'url': f"{self.token.hostName}/marketSegments/{record.get('marketSegmentId')}/subMarketSegments"
             }
         ]
         try:
-            logging.info(f"Extracting reservation {record.get('id')} additional details.")
+            logging.debug(f"Extracting reservation {record.get('id')} additional details.")
+            threads = []
+            with ThreadPoolExecutor(max_workers=4) as executor:
+                for item in feature_list:
+                    threads.append(executor.submit(self._retrieve_data, record.get("id"), item.get("name"), item.get("url")))
 
-            for item in feature_list:
-                _url = f"{self.token.hostName}/reservations/{record.get('id')}/{item.get('url')}"
-                record[item.get('name')] = self._retrieve_data(record.get("id"), item.get("name"), _url)
-
-            _field = 'reservationTypes'
-            _url = f"{self.token.hostName}/reservationTypes/{record.get('resTypeId')}"
-            record[_field] = self._retrieve_data(record.get("id"), _field, _url)
-
-            _field = 'marketSegment'
-            _url = f"{self.token.hostName}/marketSegments/{record.get('marketSegmentId')}"
-            record[_field] = self._retrieve_data(record.get("id"), _field, _url)
-
-            _field = 'subMarketSegment'
-            _url = f"{self.token.hostName}/marketSegments/{record.get('marketSegmentId')}/subMarketSegments"
-            record[_field] = [item for item in self._retrieve_data(record.get("id"), _field, _url) if
-                              item['id'] == record.get('subMarketSegmentId')]
+                for th in as_completed(threads):
+                    key, value = th.result()
+                    if key == 'subMarketSegment':
+                        value = next((x for x in value if record['subMarketSegmentId'] == x['id']), None)
+                    record[key] = value
 
             return record
         except Exception as e:
@@ -391,24 +415,44 @@ class ReservationSync:
         try:
             guestId = guest.get('id')
 
-            _field = 'accounts'
-            _url = f"{self.token.hostName}/guests/{guestId}/accounts"
-            guest[_field] = self._retrieve_data(guestId, _field, _url)
+            _fields = [
+                {
+                    'name': 'accounts',
+                    'url': f"{self.token.hostName}/guests/{guestId}/accounts"
+                },
+                {
+                    'name': 'contracts',
+                    'url': f"{self.token.hostName}/guests/{guestId}/contracts"
+                },
+                {
+                    'name': 'mealCard',
+                    'url': f'{self.token.hostName}/guests/{guestId}/mealCard'
+                }
+            ]
 
-            _field = 'contracts'
-            _url = f"{self.token.hostName}/guests/{guestId}/contracts"
-            guest[_field] = self._retrieve_data(guestId, _field, _url)
+            threads = []
+            with ThreadPoolExecutor(max_workers=4) as executor:
+                for item in _fields:
+                    threads.append(
+                        executor.submit(self._retrieve_data, guestId, item.get("name"), item.get("url")))
+
+                for th in as_completed(threads):
+                    key, value = th.result()
+                    guest[key] = value
         except Exception as e:
-            logging.error(logging.error(f'Guest profile not found : {guestId}'))
+            logging.error(logging.error(f'Guest profile additional details not found : {guestId}'))
             logging.error(e)
         finally:
             return guest
 
     def _get_guest_list(self, record):
-        logging.info(f'Processing guest list of size: {len(record.get("guests"))}')
+        logging.debug(f'Processing guest list of size: {len(record.get("guests"))}')
         _guest_list = list()
         try:
             for guest in record.get('guests'):
+                # loop = asyncio.get_event_loop()
+                # future = asyncio.ensure_future(self._extract_guest_profile(guest))
+                # loop.run_until_complete(future)
                 _guest_profile = _transform_guest_profile(self._extract_guest_profile(guest))
                 self.mongoV2.save(_guest_profile)
                 _guest_list.append(ReservationGuestShort(
@@ -436,7 +480,7 @@ class ReservationSync:
     def _get_transaction_info(self, resvId):
         result = None
         try:
-            logging.info(f'Extracting transactions for reservation: {resvId}')
+            logging.debug(f'Extracting transactions for reservation: {resvId}')
             header = {
                 'Content-Type': 'application/json',
                 'authtoken': self.token.key
@@ -462,7 +506,7 @@ class ReservationSync:
 
     def transform_reservation(self, record):
         try:
-            logging.info(f'Transforming reservation {record.get("id")}')
+            logging.debug(f'Transforming reservation {record.get("id")}')
             reservationV2 = ReservationModelV2(
                 _id=str(record.get('id')),
                 orgId=self.orgId,
@@ -528,10 +572,10 @@ class ReservationSync:
                     'businessSegmentId': record.get('businessSegmentId'),
                 },
                 reservationTypes={
-                    'id': record.get('reservationTypes').get('id'),
-                    'name': record.get('reservationTypes').get('name'),
+                    'id': record.get('reservationTypes').get('id') if record.get('reservationTypes') else None,
+                    'name': record.get('reservationTypes').get('name') if record.get('reservationTypes') else None,
                     'type': record.get('reservationType'),
-                    'inactive': record.get('reservationTypes').get('inactive'),
+                    'inactive': record.get('reservationTypes').get('inactive') if record.get('reservationTypes') else None,
                     'fixedRes': record.get('fixedRes'),
                     'fixedResReasonId': record.get('fixedResReasonId'),
                 },
@@ -711,13 +755,12 @@ class ReservationSync:
         }
 
         if not validate_key(self.token.validity):
-            logging.info('Refreshing RMS token as it is about to expire')
+            logging.debug('Refreshing RMS token as it is about to expire')
             self.token = get_key(self.orgId)
 
         record = self.extract_reservation(record)
         if record:
-            record = self.extract_reservation_details(record)
-            record = self.transform_reservation(record)
+            record = self.transform_reservation(self.extract_reservation_details(record))
             self.mongoV2.save(record)
             # self._add_reservation_to_alvie(record)
 
